@@ -114,18 +114,29 @@ class LockManager(CleanupBase):
             except OSError:
                 pass
 
+            # This value was chosen based on the py-filelock package:
+            # https://github.com/tox-dev/py-filelock/blob/a6c8fabc4192fa7a4ae19b1875ee842ec5eb4f61/src/filelock/_api.py#L113
+            wait_time_seconds = 0.05
+
+            # Warn every 10 seconds if not able to acquire lock
+            warn_attempts = int(10/wait_time_seconds)
+
+            # Exit after 60 seconds if not able to acquire lock
+            exit_attempts = int(60/wait_time_seconds)
+
             from time import sleep
-            sleep(1)
+            sleep(wait_time_seconds)
 
             attempts += 1
 
-            if attempts > 10:
+            if attempts % warn_attempts == 0:
                 from warnings import warn
                 warn("could not obtain lock -- "
                         f"delete '{self.lock_file}' if necessary",
                         stacklevel=1 + stacklevel)
-            if attempts > 3 * 60:
-                raise RuntimeError("waited more than three minutes "
+
+            if attempts > exit_attempts:
+                raise RuntimeError("waited more than one minute "
                         f"on the lock file '{self.lock_file}' "
                         "-- something is wrong")
 
@@ -261,6 +272,11 @@ class KeyBuilder:
 
     # {{{ updaters
 
+    @staticmethod
+    def update_for_type(key_hash, key):
+        key_hash.update(
+            f"{key.__module__}.{key.__qualname__}.{key.__name__}".encode("utf-8"))
+
     @staticmethod
     def update_for_int(key_hash, key):
         sz = 8
